/*
 * (C) Copyright 2011
 * CompuLab Ltd. <www.compulab.co.il>
 * Ilya Ledvich <ilya@compulab.co.il>
 * Igor Grinberg <grinberg@compulab.co.il>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.
 */

#include <common.h>
#include <asm/arch/bits.h>
#include <asm/arch/mux.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/sys_info.h>
#include <asm/arch/mem.h>

#include "common.h"
#include "prcm.h"
#include "memory.h"
#include "mux.h"
#include "board.h"

/*
 * DPLL(1-4) PARAM TABLES
 * Each of the tables has (ordered) M, N, FREQSEL, M2 values defined
 * for OPP (1.2V).
 * The values are defined for all possible sysclk and for ES1 and ES2.
 */
static struct dpll_param cm_t3517_mpu_dpll_param[] = {
	/* 12MHz */
	{0x0FE, 0x07, 0x05, 0x01}, /* ES1 */
	{0x12C, 0x05, 0x07, 0x01}, /* ES2 */
	/* 13MHz */
	{0x17D, 0x0C, 0x03, 0x01}, /* ES1 */
	{0x258, 0x0C, 0x03, 0x01}, /* ES2 */
	/* 19.2MHz */
	{0x179, 0x12, 0x04, 0x01}, /* ES1 */
	{0x271, 0x13, 0x03, 0x01}, /* ES2 */
	/* 26MHz */
	{0x17D, 0x19, 0x03, 0x01}, /* ES1 */
	{0x12C, 0x0C, 0x07, 0x01}, /* ES2 */
	/* 38.4MHz */
	{0x1FA, 0x32, 0x03, 0x01}, /* ES1 */
	{0x271, 0x27, 0x03, 0x01}  /* ES2 */
};

static struct dpll_param cm_t3517_core_dpll_param[] = {
	/* 12MHz */
	{0x19F, 0x0E, 0x03, 0x01}, /* ES1 */
	{0x0A6, 0x05, 0x07, 0x01}, /* ES2 */
	/* 13MHz */
	{0x1B2, 0x10, 0x03, 0x01}, /* ES1 */
	{0x14C, 0x0C, 0x03, 0x01}, /* ES2 */
	/* 19.2MHz */
	{0x19F, 0x17, 0x03, 0x01}, /* ES1 */
	{0x19F, 0x17, 0x03, 0x01}, /* ES2 */
	/* 26MHz */
	{0x1B2, 0x21, 0x03, 0x01}, /* ES1 */
	{0x0A6, 0x0C, 0x07, 0x01}, /* ES2 */
	/* 38.4MHz */
	{0x19F, 0x2F, 0x03, 0x01}, /* ES1 */
	{0x19F, 0x2F, 0x03, 0x01}  /* ES2 */
};

/* PER DPLL values are same for both ES1 and ES2 */
static struct dpll_param cm_t3517_per_dpll_param[] = {
	/* 12MHz */
	{0x0D8, 0x05, 0x07, 0x09},
	/* 13MHz */
	{0x1B0, 0x0C, 0x03, 0x09},
	/* 19.2MHz */
	{0x0E1, 0x09, 0x07, 0x09},
	/* 26MHz */
	{0x0D8, 0x0C, 0x07, 0x09},
	/* 38.4MHz */
	{0x0E1, 0x13, 0x07, 0x09}
};

static u32 cm_t3517_nand_gpmc_config[GPMC_MAX_REG] = {
	M_NAND_GPMC_CONFIG1,
	M_NAND_GPMC_CONFIG2,
	M_NAND_GPMC_CONFIG3,
	M_NAND_GPMC_CONFIG4,
	M_NAND_GPMC_CONFIG5,
	M_NAND_GPMC_CONFIG6,
	0
};

int cpu_is_am35xx(void)
{
	return is_cpu_family() == CPU_AM35XX;
}

static void cm_t3517_set_muxconf_regs(void)
{
	set_sdrc_muxconf_regs();
	set_gpmc_muxconf_regs();

	/* GPMC */
	MUX_VAL(CP(GPMC_nCS0),		(IDIS | PTU | EN  | M0));
	MUX_VAL(CP(GPMC_nCS4),		(IDIS | PTU | EN  | M0));
	MUX_VAL(CP(GPMC_WAIT0),		(IEN  | PTU | EN  | M0));
	/* RTC V3020 - disable the chip until driver is loaded */
	MUX_VAL(CP(UART3_CTS_RCTX),	(IEN  | PTU | EN  | M4)); /*RTC nCS*/
	MUX_VAL(CP(McBSP_CLKS),		(IEN  | PTD | EN  | M4)); /*RTC ECS*/
	/* Console */
	MUX_VAL(CP(UART3_TX_IRTX),	(IDIS | PTD | DIS | M0)); /*UART3_TX*/
	MUX_VAL(CP(UART3_RX_IRRX),	(IEN  | PTD | DIS | M0)); /*UART3_RX*/
	/* MMC1 */
	MUX_VAL(CP(MMC1_CLK),		(IEN  | PTU | EN  | M0));
	MUX_VAL(CP(MMC1_CMD),		(IEN  | PTU | DIS | M0));
	MUX_VAL(CP(MMC1_DAT0),		(IEN  | PTU | DIS | M0));
	MUX_VAL(CP(MMC1_DAT1),		(IEN  | PTU | DIS | M0));
	MUX_VAL(CP(MMC1_DAT2),		(IEN  | PTU | DIS | M0));
	MUX_VAL(CP(MMC1_DAT3),		(IEN  | PTU | DIS | M0));

	MUX_VAL(CP(SYS_CLKOUT2),	(IDIS | PTD | DIS | M4)); /*GPIO186*/
	MUX_VAL(CP(sdrc_cke0),		(IDIS | PTU | EN  | M0));
	MUX_VAL(CP(sdrc_cke1),		(IDIS | PTU | EN  | M0));
}

static void cm_t3517_per_clocks_enable(void)
{
	/* Enable cm-t3517 specific peripheral clocks */
	sr32(OMAP34XX_CTRL_BASE + 0x59C, 0, 4, 0xF);
	sr32(OMAP34XX_CTRL_BASE + 0x59C, 8, 3, 0x7);

	/* bring cpgmac out of reset */
	sr32(OMAP34XX_CTRL_BASE + 0x598, 1, 1, 0x1);

	per_clocks_enable();
}

static void cm_t3517_set_board_data(struct board_data *data)
{
	if (!data)
		hang();

	data->board_name		= "CM-T3517";
	data->watchdog_init		= watchdog_init;
	data->try_unlock_memory		= try_unlock_memory;
	data->set_muxconf_regs		= cm_t3517_set_muxconf_regs;
	data->prcm_init			= prcm_init;
	data->per_clocks_enable		= cm_t3517_per_clocks_enable;
	data->setup_memory_controller	= setup_emif4_memory_controller;
	data->core_dpll_param		= cm_t3517_core_dpll_param;
	data->per_dpll_param		= cm_t3517_per_dpll_param;
	data->mpu_dpll_param		= cm_t3517_mpu_dpll_param;
	data->nand_gpmc_config		= cm_t3517_nand_gpmc_config;
}

int cm_t3517_probe(struct board_data *data)
{
	if (!cpu_is_am35xx())
		return 1;

	cm_t3517_set_board_data(data);
	return 0;
}

